# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    webhook_list_params,
    webhook_create_params,
    webhook_update_params,
    webhook_set_active_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.webhook import Webhook
from ..types.webhook_create_response import WebhookCreateResponse
from ..types.webhook_update_response import WebhookUpdateResponse
from ..types.webhook_set_active_response import WebhookSetActiveResponse
from ..types.m3ter_signed_credentials_request_param import M3terSignedCredentialsRequestParam

__all__ = ["WebhooksResource", "AsyncWebhooksResource"]


class WebhooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return WebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return WebhooksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        credentials: M3terSignedCredentialsRequestParam,
        description: str,
        name: str,
        url: str,
        active: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookCreateResponse:
        """This endpoint creates a new webhook destination.

        A webhook destination is a URL
        where webhook payloads will be sent.

        Args:
          credentials: This schema defines the credentials required for m3ter request signing.

          description

          name

          url: The URL to which the webhook requests will be sent.

          active

          code

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/integrationdestinations/webhooks",
            body=maybe_transform(
                {
                    "credentials": credentials,
                    "description": description,
                    "name": name,
                    "url": url,
                    "active": active,
                    "code": code,
                    "version": version,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Webhook:
        """
        Retrieve the webhook Destination for the UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        credentials: M3terSignedCredentialsRequestParam,
        description: str,
        name: str,
        url: str,
        active: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookUpdateResponse:
        """
        Update a destination to be used for a webhook.

        Args:
          credentials: This schema defines the credentials required for m3ter request signing.

          description

          name

          url: The URL to which the webhook requests will be sent.

          active

          code

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            body=maybe_transform(
                {
                    "credentials": credentials,
                    "description": description,
                    "name": name,
                    "url": url,
                    "active": active,
                    "code": code,
                    "version": version,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[Webhook]:
        """
        Retrieve a list of all Destinations created in the Organization.

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of WebhookIntegrations to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/integrationdestinations/webhooks",
            page=SyncCursor[Webhook],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    webhook_list_params.WebhookListParams,
                ),
            ),
            model=Webhook,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Webhook:
        """
        This endpoint deletes a specific webhook destination identified by its UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    def set_active(
        self,
        id: str,
        *,
        org_id: str | None = None,
        active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSetActiveResponse:
        """
        Set the `active` status on a webhook integration destination.

        Use this endpoint to activate or deactivate a webhook integration destination.
        It toggles the `active` status of the specific wehbook destination with the
        given ID.

        Args:
          active: active status of the webhook

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}/active",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"active": active}, webhook_set_active_params.WebhookSetActiveParams),
            ),
            cast_to=WebhookSetActiveResponse,
        )


class AsyncWebhooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncWebhooksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        credentials: M3terSignedCredentialsRequestParam,
        description: str,
        name: str,
        url: str,
        active: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookCreateResponse:
        """This endpoint creates a new webhook destination.

        A webhook destination is a URL
        where webhook payloads will be sent.

        Args:
          credentials: This schema defines the credentials required for m3ter request signing.

          description

          name

          url: The URL to which the webhook requests will be sent.

          active

          code

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/integrationdestinations/webhooks",
            body=await async_maybe_transform(
                {
                    "credentials": credentials,
                    "description": description,
                    "name": name,
                    "url": url,
                    "active": active,
                    "code": code,
                    "version": version,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Webhook:
        """
        Retrieve the webhook Destination for the UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        credentials: M3terSignedCredentialsRequestParam,
        description: str,
        name: str,
        url: str,
        active: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookUpdateResponse:
        """
        Update a destination to be used for a webhook.

        Args:
          credentials: This schema defines the credentials required for m3ter request signing.

          description

          name

          url: The URL to which the webhook requests will be sent.

          active

          code

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            body=await async_maybe_transform(
                {
                    "credentials": credentials,
                    "description": description,
                    "name": name,
                    "url": url,
                    "active": active,
                    "code": code,
                    "version": version,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Webhook, AsyncCursor[Webhook]]:
        """
        Retrieve a list of all Destinations created in the Organization.

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of WebhookIntegrations to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/integrationdestinations/webhooks",
            page=AsyncCursor[Webhook],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    webhook_list_params.WebhookListParams,
                ),
            ),
            model=Webhook,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Webhook:
        """
        This endpoint deletes a specific webhook destination identified by its UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    async def set_active(
        self,
        id: str,
        *,
        org_id: str | None = None,
        active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSetActiveResponse:
        """
        Set the `active` status on a webhook integration destination.

        Use this endpoint to activate or deactivate a webhook integration destination.
        It toggles the `active` status of the specific wehbook destination with the
        given ID.

        Args:
          active: active status of the webhook

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/integrationdestinations/webhooks/{id}/active",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"active": active}, webhook_set_active_params.WebhookSetActiveParams),
            ),
            cast_to=WebhookSetActiveResponse,
        )


class WebhooksResourceWithRawResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = to_raw_response_wrapper(
            webhooks.create,
        )
        self.retrieve = to_raw_response_wrapper(
            webhooks.retrieve,
        )
        self.update = to_raw_response_wrapper(
            webhooks.update,
        )
        self.list = to_raw_response_wrapper(
            webhooks.list,
        )
        self.delete = to_raw_response_wrapper(
            webhooks.delete,
        )
        self.set_active = to_raw_response_wrapper(
            webhooks.set_active,
        )


class AsyncWebhooksResourceWithRawResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = async_to_raw_response_wrapper(
            webhooks.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            webhooks.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            webhooks.update,
        )
        self.list = async_to_raw_response_wrapper(
            webhooks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            webhooks.delete,
        )
        self.set_active = async_to_raw_response_wrapper(
            webhooks.set_active,
        )


class WebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = to_streamed_response_wrapper(
            webhooks.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            webhooks.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            webhooks.update,
        )
        self.list = to_streamed_response_wrapper(
            webhooks.list,
        )
        self.delete = to_streamed_response_wrapper(
            webhooks.delete,
        )
        self.set_active = to_streamed_response_wrapper(
            webhooks.set_active,
        )


class AsyncWebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = async_to_streamed_response_wrapper(
            webhooks.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            webhooks.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            webhooks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            webhooks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            webhooks.delete,
        )
        self.set_active = async_to_streamed_response_wrapper(
            webhooks.set_active,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CurrencyListParams"]


class CurrencyListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    archived: bool
    """Filter by archived flag.

    A True / False flag indicating whether to return Currencies that are archived
    _(obsolete)_.

    - TRUE - return archived Currencies.
    - FALSE - archived Currencies are not returned.
    """

    codes: List[str]
    """
    An optional parameter to retrieve specific Currencies based on their short
    codes.
    """

    ids: List[str]
    """
    An optional parameter to filter the list based on specific Currency unique
    identifiers (UUIDs).
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Currencies in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of Currencies to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import PricingResponse
from tests.utils import assert_matches_type
from m3ter._utils import parse_datetime
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPricings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        pricing = client.pricings.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        pricing = client.pricings.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            aggregation_id="aggregationId",
            code="JS!?Q0]r] ]$]",
            compound_aggregation_id="compoundAggregationId",
            cumulative=True,
            description="description",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            minimum_spend=0,
            minimum_spend_bill_in_advance=True,
            minimum_spend_description="minimumSpendDescription",
            overage_pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            plan_id="planId",
            plan_template_id="planTemplateId",
            segment={"foo": "string"},
            tiers_span_plan=True,
            type="DEBIT",
            version=0,
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.pricings.with_raw_response.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.pricings.with_streaming_response.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.pricings.with_raw_response.create(
                org_id="",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        pricing = client.pricings.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.pricings.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.pricings.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.pricings.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.pricings.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        pricing = client.pricings.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        pricing = client.pricings.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            aggregation_id="aggregationId",
            code="JS!?Q0]r] ]$]",
            compound_aggregation_id="compoundAggregationId",
            cumulative=True,
            description="description",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            minimum_spend=0,
            minimum_spend_bill_in_advance=True,
            minimum_spend_description="minimumSpendDescription",
            overage_pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            plan_id="planId",
            plan_template_id="planTemplateId",
            segment={"foo": "string"},
            tiers_span_plan=True,
            type="DEBIT",
            version=0,
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.pricings.with_raw_response.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.pricings.with_streaming_response.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.pricings.with_raw_response.update(
                id="id",
                org_id="",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.pricings.with_raw_response.update(
                id="",
                org_id="orgId",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        pricing = client.pricings.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        pricing = client.pricings.list(
            org_id="orgId",
            date="date",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            plan_id="planId",
            plan_template_id="planTemplateId",
        )
        assert_matches_type(SyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.pricings.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = response.parse()
        assert_matches_type(SyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.pricings.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = response.parse()
            assert_matches_type(SyncCursor[PricingResponse], pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.pricings.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        pricing = client.pricings.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.pricings.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.pricings.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.pricings.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.pricings.with_raw_response.delete(
                id="",
                org_id="orgId",
            )


class TestAsyncPricings:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            aggregation_id="aggregationId",
            code="JS!?Q0]r] ]$]",
            compound_aggregation_id="compoundAggregationId",
            cumulative=True,
            description="description",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            minimum_spend=0,
            minimum_spend_bill_in_advance=True,
            minimum_spend_description="minimumSpendDescription",
            overage_pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            plan_id="planId",
            plan_template_id="planTemplateId",
            segment={"foo": "string"},
            tiers_span_plan=True,
            type="DEBIT",
            version=0,
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.pricings.with_raw_response.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = await response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.pricings.with_streaming_response.create(
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = await response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.pricings.with_raw_response.create(
                org_id="",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.pricings.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = await response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.pricings.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = await response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.pricings.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.pricings.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            aggregation_id="aggregationId",
            code="JS!?Q0]r] ]$]",
            compound_aggregation_id="compoundAggregationId",
            cumulative=True,
            description="description",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            minimum_spend=0,
            minimum_spend_bill_in_advance=True,
            minimum_spend_description="minimumSpendDescription",
            overage_pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                    "id": "id",
                    "credit_type_id": "creditTypeId",
                }
            ],
            plan_id="planId",
            plan_template_id="planTemplateId",
            segment={"foo": "string"},
            tiers_span_plan=True,
            type="DEBIT",
            version=0,
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.pricings.with_raw_response.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = await response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.pricings.with_streaming_response.update(
            id="id",
            org_id="orgId",
            pricing_bands=[
                {
                    "fixed_price": 0,
                    "lower_limit": 0,
                    "unit_price": 0,
                }
            ],
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = await response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.pricings.with_raw_response.update(
                id="id",
                org_id="",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.pricings.with_raw_response.update(
                id="",
                org_id="orgId",
                pricing_bands=[
                    {
                        "fixed_price": 0,
                        "lower_limit": 0,
                        "unit_price": 0,
                    }
                ],
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.list(
            org_id="orgId",
            date="date",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            plan_id="planId",
            plan_template_id="planTemplateId",
        )
        assert_matches_type(AsyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.pricings.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = await response.parse()
        assert_matches_type(AsyncCursor[PricingResponse], pricing, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.pricings.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = await response.parse()
            assert_matches_type(AsyncCursor[PricingResponse], pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.pricings.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        pricing = await async_client.pricings.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.pricings.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pricing = await response.parse()
        assert_matches_type(PricingResponse, pricing, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.pricings.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pricing = await response.parse()
            assert_matches_type(PricingResponse, pricing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.pricings.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.pricings.with_raw_response.delete(
                id="",
                org_id="orgId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["FileUploadGenerateUploadURLParams"]


class FileUploadGenerateUploadURLParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    content_type: Required[Annotated[str, PropertyInfo(alias="contentType")]]
    """
    The media type of the entity body sent, for example:
    `"contentType":"text/json"`.

    **NOTE:** Currently only a JSON formatted file type is supported by the File
    Upload Service.
    """

    file_name: Required[Annotated[str, PropertyInfo(alias="fileName")]]
    """The name of the measurements file to be uploaded."""

    content_length: Annotated[int, PropertyInfo(alias="contentLength")]
    """The size of the body in bytes.

    For example: `"contentLength": 485`, where 485 is the size in bytes of the file
    to upload.

    **NOTE:** Required.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["EventResponse"]


class EventResponse(BaseModel):
    id: str
    """The uniqie identifier (UUID) of the Event."""

    dt_actioned: datetime = FieldInfo(alias="dtActioned")
    """When an Event was actioned. It follows the ISO 8601 date and time format.

    You can action an Event to indicate that it has been followed up and resolved -
    this is useful when dealing with integration error Events or ingest failure
    Events.
    """

    event_name: str = FieldInfo(alias="eventName")
    """The name of the Event as it is registered in the system.

    This name is used to categorize and trigger associated actions.
    """

    event_time: datetime = FieldInfo(alias="eventTime")
    """The time when the Event was triggered, using the ISO 8601 date and time format."""

    m3ter_event: object = FieldInfo(alias="m3terEvent")
    """The Data Transfer Object (DTO) containing the details of the Event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CompoundAggregationListParams"]


class CompoundAggregationListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    codes: List[str]
    """
    An optional parameter to retrieve specific CompoundAggregations based on their
    short codes.
    """

    ids: List[str]
    """
    An optional parameter to retrieve specific CompoundAggregations based on their
    unique identifiers (UUIDs).
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of CompoundAggregations in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of CompoundAggregations to retrieve per page."""

    product_id: Annotated[List[str], PropertyInfo(alias="productId")]
    """
    An optional parameter to filter the CompoundAggregations based on specific
    Product unique identifiers (UUIDs).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union

import httpx

from ..types import plan_list_params, plan_create_params, plan_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.plan_response import PlanResponse

__all__ = ["PlansResource", "AsyncPlansResource"]


class PlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return PlansResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        plan_template_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        bespoke: bool | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_accounting_product_id: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        ordinal: int | NotGiven = NOT_GIVEN,
        standing_charge: float | NotGiven = NOT_GIVEN,
        standing_charge_accounting_product_id: str | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        standing_charge_description: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Create a new Plan.

        Args:
          code: Unique short code reference for the Plan.

          name: Descriptive name for the Plan.

          plan_template_id: UUID of the PlanTemplate the Plan belongs to.

          account_id: _(Optional)_. Used to specify an Account for which the Plan will be a
              custom/bespoke Plan:

              - Use when first creating a Plan.
              - A custom/bespoke Plan can only be attached to the specified Account.
              - Once created, a custom/bespoke Plan cannot be updated to be made a
                custom/bespoke Plan for a different Account.

          bespoke: TRUE/FALSE flag indicating whether the plan is a custom/bespoke Plan for a
              particular Account:

              - When creating a Plan, use the `accountId` request parameter to specify the
                Account for which the Plan will be custom/bespoke.
              - A custom/bespoke Plan can only be attached to the specified Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          minimum_spend: The product minimum spend amount per billing cycle for end customer Accounts on
              a priced Plan.

              _(Optional)_. Overrides PlanTemplate value.

          minimum_spend_accounting_product_id: Optional Product ID this plan's minimum spend should be attributed to for
              accounting purposes

          minimum_spend_bill_in_advance: When TRUE, minimum spend is billed at the start of each billing period.

              When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          ordinal: Assigns a rank or position to the Plan in your order of pricing plans - lower
              numbers represent more basic pricing plans; higher numbers represent more
              premium pricing plans.

              _(Optional)_. Overrides PlanTemplate value.

              **NOTE: DEPRECATED** - do not use.

          standing_charge: The standing charge applied to bills for end customers. This is prorated.

              _(Optional)_. Overrides PlanTemplate value.

          standing_charge_accounting_product_id: Optional Product ID this plan's standing charge should be attributed to for
              accounting purposes

          standing_charge_bill_in_advance: When TRUE, standing charge is billed at the start of each billing period.

              When FALSE, standing charge is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for standing charge
              billing in arrears/in advance.

          standing_charge_description: Standing charge description _(displayed on the bill line item)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/plans",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "plan_template_id": plan_template_id,
                    "account_id": account_id,
                    "bespoke": bespoke,
                    "custom_fields": custom_fields,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_accounting_product_id": minimum_spend_accounting_product_id,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "ordinal": ordinal,
                    "standing_charge": standing_charge,
                    "standing_charge_accounting_product_id": standing_charge_accounting_product_id,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "standing_charge_description": standing_charge_description,
                    "version": version,
                },
                plan_create_params.PlanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Retrieve the Plan with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/plans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        plan_template_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        bespoke: bool | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_accounting_product_id: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        ordinal: int | NotGiven = NOT_GIVEN,
        standing_charge: float | NotGiven = NOT_GIVEN,
        standing_charge_accounting_product_id: str | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        standing_charge_description: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Update the Plan with the given UUID.

        **Note:** If you have created Custom Fields for a Plan, when you use this
        endpoint to update the Plan use the `customFields` parameter to preserve those
        Custom Fields. If you omit them from the update request, they will be lost.

        Args:
          code: Unique short code reference for the Plan.

          name: Descriptive name for the Plan.

          plan_template_id: UUID of the PlanTemplate the Plan belongs to.

          account_id: _(Optional)_. Used to specify an Account for which the Plan will be a
              custom/bespoke Plan:

              - Use when first creating a Plan.
              - A custom/bespoke Plan can only be attached to the specified Account.
              - Once created, a custom/bespoke Plan cannot be updated to be made a
                custom/bespoke Plan for a different Account.

          bespoke: TRUE/FALSE flag indicating whether the plan is a custom/bespoke Plan for a
              particular Account:

              - When creating a Plan, use the `accountId` request parameter to specify the
                Account for which the Plan will be custom/bespoke.
              - A custom/bespoke Plan can only be attached to the specified Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          minimum_spend: The product minimum spend amount per billing cycle for end customer Accounts on
              a priced Plan.

              _(Optional)_. Overrides PlanTemplate value.

          minimum_spend_accounting_product_id: Optional Product ID this plan's minimum spend should be attributed to for
              accounting purposes

          minimum_spend_bill_in_advance: When TRUE, minimum spend is billed at the start of each billing period.

              When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          ordinal: Assigns a rank or position to the Plan in your order of pricing plans - lower
              numbers represent more basic pricing plans; higher numbers represent more
              premium pricing plans.

              _(Optional)_. Overrides PlanTemplate value.

              **NOTE: DEPRECATED** - do not use.

          standing_charge: The standing charge applied to bills for end customers. This is prorated.

              _(Optional)_. Overrides PlanTemplate value.

          standing_charge_accounting_product_id: Optional Product ID this plan's standing charge should be attributed to for
              accounting purposes

          standing_charge_bill_in_advance: When TRUE, standing charge is billed at the start of each billing period.

              When FALSE, standing charge is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for standing charge
              billing in arrears/in advance.

          standing_charge_description: Standing charge description _(displayed on the bill line item)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/plans/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "plan_template_id": plan_template_id,
                    "account_id": account_id,
                    "bespoke": bespoke,
                    "custom_fields": custom_fields,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_accounting_product_id": minimum_spend_accounting_product_id,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "ordinal": ordinal,
                    "standing_charge": standing_charge,
                    "standing_charge_accounting_product_id": standing_charge_accounting_product_id,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "standing_charge_description": standing_charge_description,
                    "version": version,
                },
                plan_update_params.PlanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[PlanResponse]:
        """
        Retrieve a list of Plans that can be filtered by Product, Account, or Plan ID.

        Args:
          account_id: List of Account IDs the Plan belongs to.

          ids: List of Plan IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of Plans to retrieve per page.

          product_id: UUID of the Product to retrieve Plans for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/plans",
            page=SyncCursor[PlanResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Delete the Plan with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/plans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )


class AsyncPlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncPlansResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        plan_template_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        bespoke: bool | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_accounting_product_id: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        ordinal: int | NotGiven = NOT_GIVEN,
        standing_charge: float | NotGiven = NOT_GIVEN,
        standing_charge_accounting_product_id: str | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        standing_charge_description: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Create a new Plan.

        Args:
          code: Unique short code reference for the Plan.

          name: Descriptive name for the Plan.

          plan_template_id: UUID of the PlanTemplate the Plan belongs to.

          account_id: _(Optional)_. Used to specify an Account for which the Plan will be a
              custom/bespoke Plan:

              - Use when first creating a Plan.
              - A custom/bespoke Plan can only be attached to the specified Account.
              - Once created, a custom/bespoke Plan cannot be updated to be made a
                custom/bespoke Plan for a different Account.

          bespoke: TRUE/FALSE flag indicating whether the plan is a custom/bespoke Plan for a
              particular Account:

              - When creating a Plan, use the `accountId` request parameter to specify the
                Account for which the Plan will be custom/bespoke.
              - A custom/bespoke Plan can only be attached to the specified Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          minimum_spend: The product minimum spend amount per billing cycle for end customer Accounts on
              a priced Plan.

              _(Optional)_. Overrides PlanTemplate value.

          minimum_spend_accounting_product_id: Optional Product ID this plan's minimum spend should be attributed to for
              accounting purposes

          minimum_spend_bill_in_advance: When TRUE, minimum spend is billed at the start of each billing period.

              When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          ordinal: Assigns a rank or position to the Plan in your order of pricing plans - lower
              numbers represent more basic pricing plans; higher numbers represent more
              premium pricing plans.

              _(Optional)_. Overrides PlanTemplate value.

              **NOTE: DEPRECATED** - do not use.

          standing_charge: The standing charge applied to bills for end customers. This is prorated.

              _(Optional)_. Overrides PlanTemplate value.

          standing_charge_accounting_product_id: Optional Product ID this plan's standing charge should be attributed to for
              accounting purposes

          standing_charge_bill_in_advance: When TRUE, standing charge is billed at the start of each billing period.

              When FALSE, standing charge is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for standing charge
              billing in arrears/in advance.

          standing_charge_description: Standing charge description _(displayed on the bill line item)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/plans",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "plan_template_id": plan_template_id,
                    "account_id": account_id,
                    "bespoke": bespoke,
                    "custom_fields": custom_fields,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_accounting_product_id": minimum_spend_accounting_product_id,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "ordinal": ordinal,
                    "standing_charge": standing_charge,
                    "standing_charge_accounting_product_id": standing_charge_accounting_product_id,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "standing_charge_description": standing_charge_description,
                    "version": version,
                },
                plan_create_params.PlanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Retrieve the Plan with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/plans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        plan_template_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        bespoke: bool | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_accounting_product_id: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        ordinal: int | NotGiven = NOT_GIVEN,
        standing_charge: float | NotGiven = NOT_GIVEN,
        standing_charge_accounting_product_id: str | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        standing_charge_description: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Update the Plan with the given UUID.

        **Note:** If you have created Custom Fields for a Plan, when you use this
        endpoint to update the Plan use the `customFields` parameter to preserve those
        Custom Fields. If you omit them from the update request, they will be lost.

        Args:
          code: Unique short code reference for the Plan.

          name: Descriptive name for the Plan.

          plan_template_id: UUID of the PlanTemplate the Plan belongs to.

          account_id: _(Optional)_. Used to specify an Account for which the Plan will be a
              custom/bespoke Plan:

              - Use when first creating a Plan.
              - A custom/bespoke Plan can only be attached to the specified Account.
              - Once created, a custom/bespoke Plan cannot be updated to be made a
                custom/bespoke Plan for a different Account.

          bespoke: TRUE/FALSE flag indicating whether the plan is a custom/bespoke Plan for a
              particular Account:

              - When creating a Plan, use the `accountId` request parameter to specify the
                Account for which the Plan will be custom/bespoke.
              - A custom/bespoke Plan can only be attached to the specified Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          minimum_spend: The product minimum spend amount per billing cycle for end customer Accounts on
              a priced Plan.

              _(Optional)_. Overrides PlanTemplate value.

          minimum_spend_accounting_product_id: Optional Product ID this plan's minimum spend should be attributed to for
              accounting purposes

          minimum_spend_bill_in_advance: When TRUE, minimum spend is billed at the start of each billing period.

              When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          ordinal: Assigns a rank or position to the Plan in your order of pricing plans - lower
              numbers represent more basic pricing plans; higher numbers represent more
              premium pricing plans.

              _(Optional)_. Overrides PlanTemplate value.

              **NOTE: DEPRECATED** - do not use.

          standing_charge: The standing charge applied to bills for end customers. This is prorated.

              _(Optional)_. Overrides PlanTemplate value.

          standing_charge_accounting_product_id: Optional Product ID this plan's standing charge should be attributed to for
              accounting purposes

          standing_charge_bill_in_advance: When TRUE, standing charge is billed at the start of each billing period.

              When FALSE, standing charge is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at PlanTemplate level for standing charge
              billing in arrears/in advance.

          standing_charge_description: Standing charge description _(displayed on the bill line item)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/plans/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "plan_template_id": plan_template_id,
                    "account_id": account_id,
                    "bespoke": bespoke,
                    "custom_fields": custom_fields,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_accounting_product_id": minimum_spend_accounting_product_id,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "ordinal": ordinal,
                    "standing_charge": standing_charge,
                    "standing_charge_accounting_product_id": standing_charge_accounting_product_id,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "standing_charge_description": standing_charge_description,
                    "version": version,
                },
                plan_update_params.PlanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PlanResponse, AsyncCursor[PlanResponse]]:
        """
        Retrieve a list of Plans that can be filtered by Product, Account, or Plan ID.

        Args:
          account_id: List of Account IDs the Plan belongs to.

          ids: List of Plan IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of Plans to retrieve per page.

          product_id: UUID of the Product to retrieve Plans for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/plans",
            page=AsyncCursor[PlanResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlanResponse:
        """
        Delete the Plan with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/plans/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanResponse,
        )


class PlansResourceWithRawResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.create = to_raw_response_wrapper(
            plans.create,
        )
        self.retrieve = to_raw_response_wrapper(
            plans.retrieve,
        )
        self.update = to_raw_response_wrapper(
            plans.update,
        )
        self.list = to_raw_response_wrapper(
            plans.list,
        )
        self.delete = to_raw_response_wrapper(
            plans.delete,
        )


class AsyncPlansResourceWithRawResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.create = async_to_raw_response_wrapper(
            plans.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            plans.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            plans.update,
        )
        self.list = async_to_raw_response_wrapper(
            plans.list,
        )
        self.delete = async_to_raw_response_wrapper(
            plans.delete,
        )


class PlansResourceWithStreamingResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.create = to_streamed_response_wrapper(
            plans.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            plans.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            plans.update,
        )
        self.list = to_streamed_response_wrapper(
            plans.list,
        )
        self.delete = to_streamed_response_wrapper(
            plans.delete,
        )


class AsyncPlansResourceWithStreamingResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.create = async_to_streamed_response_wrapper(
            plans.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            plans.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            plans.update,
        )
        self.list = async_to_streamed_response_wrapper(
            plans.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            plans.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    CommitmentResponse,
    CommitmentSearchResponse,
)
from tests.utils import assert_matches_type
from m3ter._utils import parse_date, parse_datetime
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCommitments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        commitment = client.commitments.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        commitment = client.commitments.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount_first_bill=0,
            amount_pre_paid=0,
            bill_epoch=parse_date("2019-12-27"),
            billing_interval=1,
            billing_offset=0,
            billing_plan_id="billingPlanId",
            child_billing_mode="PARENT_SUMMARY",
            commitment_fee_bill_in_advance=True,
            commitment_fee_description="commitmentFeeDescription",
            commitment_usage_description="commitmentUsageDescription",
            contract_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            drawdowns_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            fee_dates=[
                {
                    "amount": 1,
                    "date": parse_date("2019-12-27"),
                    "service_period_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "service_period_start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            fees_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            line_item_types=["STANDING_CHARGE"],
            overage_description="overageDescription",
            overage_surcharge_percent=0,
            product_ids=["string"],
            separate_overage_usage=True,
            version=0,
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.create(
                org_id="",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        commitment = client.commitments.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.commitments.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        commitment = client.commitments.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        commitment = client.commitments.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount_first_bill=0,
            amount_pre_paid=0,
            bill_epoch=parse_date("2019-12-27"),
            billing_interval=1,
            billing_offset=0,
            billing_plan_id="billingPlanId",
            child_billing_mode="PARENT_SUMMARY",
            commitment_fee_bill_in_advance=True,
            commitment_fee_description="commitmentFeeDescription",
            commitment_usage_description="commitmentUsageDescription",
            contract_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            drawdowns_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            fee_dates=[
                {
                    "amount": 1,
                    "date": parse_date("2019-12-27"),
                    "service_period_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "service_period_start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            fees_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            line_item_types=["STANDING_CHARGE"],
            overage_description="overageDescription",
            overage_surcharge_percent=0,
            product_ids=["string"],
            separate_overage_usage=True,
            version=0,
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.update(
                id="id",
                org_id="",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.commitments.with_raw_response.update(
                id="",
                org_id="orgId",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        commitment = client.commitments.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        commitment = client.commitments.list(
            org_id="orgId",
            account_id="accountId",
            contract_id="contractId",
            date="date",
            end_date_end="endDateEnd",
            end_date_start="endDateStart",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            product_id="productId",
        )
        assert_matches_type(SyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(SyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(SyncCursor[CommitmentResponse], commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        commitment = client.commitments.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.commitments.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_search(self, client: M3ter) -> None:
        commitment = client.commitments.search(
            org_id="orgId",
        )
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: M3ter) -> None:
        commitment = client.commitments.search(
            org_id="orgId",
            from_document=0,
            operator="AND",
            page_size=1,
            search_query="searchQuery",
            sort_by="sortBy",
            sort_order="ASC",
        )
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: M3ter) -> None:
        response = client.commitments.with_raw_response.search(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = response.parse()
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: M3ter) -> None:
        with client.commitments.with_streaming_response.search(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = response.parse()
            assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_search(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.commitments.with_raw_response.search(
                org_id="",
            )


class TestAsyncCommitments:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount_first_bill=0,
            amount_pre_paid=0,
            bill_epoch=parse_date("2019-12-27"),
            billing_interval=1,
            billing_offset=0,
            billing_plan_id="billingPlanId",
            child_billing_mode="PARENT_SUMMARY",
            commitment_fee_bill_in_advance=True,
            commitment_fee_description="commitmentFeeDescription",
            commitment_usage_description="commitmentUsageDescription",
            contract_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            drawdowns_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            fee_dates=[
                {
                    "amount": 1,
                    "date": parse_date("2019-12-27"),
                    "service_period_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "service_period_start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            fees_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            line_item_types=["STANDING_CHARGE"],
            overage_description="overageDescription",
            overage_surcharge_percent=0,
            product_ids=["string"],
            separate_overage_usage=True,
            version=0,
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.create(
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.create(
                org_id="",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.commitments.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
            accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount_first_bill=0,
            amount_pre_paid=0,
            bill_epoch=parse_date("2019-12-27"),
            billing_interval=1,
            billing_offset=0,
            billing_plan_id="billingPlanId",
            child_billing_mode="PARENT_SUMMARY",
            commitment_fee_bill_in_advance=True,
            commitment_fee_description="commitmentFeeDescription",
            commitment_usage_description="commitmentUsageDescription",
            contract_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            drawdowns_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            fee_dates=[
                {
                    "amount": 1,
                    "date": parse_date("2019-12-27"),
                    "service_period_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "service_period_start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            fees_accounting_product_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            line_item_types=["STANDING_CHARGE"],
            overage_description="overageDescription",
            overage_surcharge_percent=0,
            product_ids=["string"],
            separate_overage_usage=True,
            version=0,
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.update(
            id="id",
            org_id="orgId",
            account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            amount=1,
            currency="x",
            end_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.update(
                id="id",
                org_id="",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.commitments.with_raw_response.update(
                id="",
                org_id="orgId",
                account_id="xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                amount=1,
                currency="x",
                end_date=parse_date("2019-12-27"),
                start_date=parse_date("2019-12-27"),
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.list(
            org_id="orgId",
            account_id="accountId",
            contract_id="contractId",
            date="date",
            end_date_end="endDateEnd",
            end_date_start="endDateStart",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            product_id="productId",
        )
        assert_matches_type(AsyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(AsyncCursor[CommitmentResponse], commitment, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(AsyncCursor[CommitmentResponse], commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(CommitmentResponse, commitment, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(CommitmentResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.commitments.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_search(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.search(
            org_id="orgId",
        )
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncM3ter) -> None:
        commitment = await async_client.commitments.search(
            org_id="orgId",
            from_document=0,
            operator="AND",
            page_size=1,
            search_query="searchQuery",
            sort_by="sortBy",
            sort_order="ASC",
        )
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncM3ter) -> None:
        response = await async_client.commitments.with_raw_response.search(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        commitment = await response.parse()
        assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncM3ter) -> None:
        async with async_client.commitments.with_streaming_response.search(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            commitment = await response.parse()
            assert_matches_type(CommitmentSearchResponse, commitment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_search(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.commitments.with_raw_response.search(
                org_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.data_exports import job_list_params
from ...types.data_exports.data_export_job_response import DataExportJobResponse
from ...types.data_exports.job_get_download_url_response import JobGetDownloadURLResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataExportJobResponse:
        """
        Retrieve an Export Job for the given UUID.

        The response returns:

        - The source type for the data exported by the Export Job: one of USAGE or
          OPERATIONAL.
        - The status of the Export Job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/dataexports/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataExportJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | NotGiven = NOT_GIVEN,
        date_created_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        schedule_id: str | NotGiven = NOT_GIVEN,
        status: Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[DataExportJobResponse]:
        """
        Retrieve a list of Export Job entities.

        Args:
          date_created_end:
              Include only Job entities created before this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start:
              Include only Job entities created on or after this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          ids: List Job entities for the given UUIDs

          next_token: nextToken for multi page retrievals

          page_size: Number of Jobs to retrieve per page

          schedule_id: List Job entities for the schedule UUID

          status: List Job entities for the status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/jobs",
            page=SyncCursor[DataExportJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                        "status": status,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=DataExportJobResponse,
        )

    def get_download_url(
        self,
        job_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JobGetDownloadURLResponse:
        """
        Returns a presigned download URL for data export file download based on the
        `jobId` provided.

        If you omit `destinationIds` when setting up your
        [Ad-Hoc data exports](https://www.m3ter.com/docs/api#tag/ExportAdHoc) or
        [Scheduled data exports](https://www.m3ter.com/docs/api#tag/ExportSchedule),
        then the data is not copied to a destination but is available for you to
        download using the returned download URL.

        **Constraints:**

        - Only valid for Export jobs ran in the past 24 hours.
        - The download URL is time-bound and is only valid for 15 minutes.

        **NOTE!** This ExportDestination endpoint is available in Beta release version.
        Beta release features are functional but may be incomplete, and there is no
        commitment at this stage to particular functionality or timelines.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            f"/organizations/{org_id}/dataexports/jobs/{job_id}/getdownloadurl",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetDownloadURLResponse,
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DataExportJobResponse:
        """
        Retrieve an Export Job for the given UUID.

        The response returns:

        - The source type for the data exported by the Export Job: one of USAGE or
          OPERATIONAL.
        - The status of the Export Job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/dataexports/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataExportJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | NotGiven = NOT_GIVEN,
        date_created_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        schedule_id: str | NotGiven = NOT_GIVEN,
        status: Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[DataExportJobResponse, AsyncCursor[DataExportJobResponse]]:
        """
        Retrieve a list of Export Job entities.

        Args:
          date_created_end:
              Include only Job entities created before this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start:
              Include only Job entities created on or after this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          ids: List Job entities for the given UUIDs

          next_token: nextToken for multi page retrievals

          page_size: Number of Jobs to retrieve per page

          schedule_id: List Job entities for the schedule UUID

          status: List Job entities for the status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/jobs",
            page=AsyncCursor[DataExportJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                        "status": status,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=DataExportJobResponse,
        )

    async def get_download_url(
        self,
        job_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JobGetDownloadURLResponse:
        """
        Returns a presigned download URL for data export file download based on the
        `jobId` provided.

        If you omit `destinationIds` when setting up your
        [Ad-Hoc data exports](https://www.m3ter.com/docs/api#tag/ExportAdHoc) or
        [Scheduled data exports](https://www.m3ter.com/docs/api#tag/ExportSchedule),
        then the data is not copied to a destination but is available for you to
        download using the returned download URL.

        **Constraints:**

        - Only valid for Export jobs ran in the past 24 hours.
        - The download URL is time-bound and is only valid for 15 minutes.

        **NOTE!** This ExportDestination endpoint is available in Beta release version.
        Beta release features are functional but may be incomplete, and there is no
        commitment at this stage to particular functionality or timelines.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            f"/organizations/{org_id}/dataexports/jobs/{job_id}/getdownloadurl",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetDownloadURLResponse,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.get_download_url = to_raw_response_wrapper(
            jobs.get_download_url,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.get_download_url = async_to_raw_response_wrapper(
            jobs.get_download_url,
        )


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_download_url = to_streamed_response_wrapper(
            jobs.get_download_url,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_download_url = async_to_streamed_response_wrapper(
            jobs.get_download_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CompoundAggregationCreateParams"]


class CompoundAggregationCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    calculation: Required[str]
    """String that represents the formula for the calculation.

    This formula determines how the CompoundAggregation value is calculated. The
    calculation can reference simple Aggregations or Custom Fields. This field is
    required when creating or updating a CompoundAggregation.

    **NOTE:** If a simple Aggregation referenced by a Compound Aggregation has a
    **Quantity per unit** defined or a **Rounding** defined, these will not be
    factored into the value used by the calculation. For example, if the simple
    Aggregation referenced has a base value of 100 and has **Quantity per unit** set
    at 10, the Compound Aggregation calculation _will use the base value of 100 not
    10_.
    """

    name: Required[str]
    """Descriptive name for the Aggregation."""

    quantity_per_unit: Required[Annotated[float, PropertyInfo(alias="quantityPerUnit")]]
    """Defines how much of a quantity equates to 1 unit.

    Used when setting the price per unit for billing purposes - if charging for
    kilobytes per second (KiBy/s) at rate of $0.25 per 500 KiBy/s, then set
    quantityPerUnit to 500 and price Plan at $0.25 per unit.

    **Note:** If `quantityPerUnit` is set to a value other than one, `rounding` is
    typically set to `"UP"`.
    """

    rounding: Required[Literal["UP", "DOWN", "NEAREST", "NONE"]]
    """
    Specifies how you want to deal with non-integer, fractional number Aggregation
    values.

    **NOTES:**

    - **NEAREST** rounds to the nearest half: 5.1 is rounded to 5, and 3.5 is
      rounded to 4.
    - Also used in combination with `quantityPerUnit`. Rounds the number of units
      after `quantityPerUnit` is applied. If you set `quantityPerUnit` to a value
      other than one, you would typically set Rounding to **UP**. For example,
      suppose you charge by kilobytes per second (KiBy/s), set `quantityPerUnit` =
      500, and set charge rate at $0.25 per unit used. If your customer used 48,900
      KiBy/s in a billing period, the charge would be 48,900 / 500 = 97.8 rounded up
      to 98 \\** 0.25 = $2.45.

    Enum: ???UP??? ???DOWN??? ???NEAREST??? ???NONE???
    """

    unit: Required[str]
    """
    User defined label for units shown for Bill line items, indicating to your
    customers what they are being charged for.
    """

    accounting_product_id: Annotated[str, PropertyInfo(alias="accountingProductId")]
    """
    Optional Product ID this Aggregation should be attributed to for accounting
    purposes
    """

    code: str
    """Code of the new Aggregation. A unique short code to identify the Aggregation."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]

    evaluate_null_aggregations: Annotated[bool, PropertyInfo(alias="evaluateNullAggregations")]
    """Boolean True / False flag:

    - **TRUE** - set to TRUE if you want to allow null values from the simple
      Aggregations referenced in the Compound Aggregation to be passed in. Simple
      Aggregations based on Meter Target Fields where no usage data is available
      will have null values.
    - **FALSE** Default.

    **Note:** If any of the simple Aggregations you reference in a Compound
    Aggregation calculation might have null values, you must set their Default Value
    to 0. This ensures that any null values passed into the Compound Aggregation are
    passed in correctly with value = 0.
    """

    product_id: Annotated[str, PropertyInfo(alias="productId")]
    """Unique identifier (UUID) of the Product the CompoundAggregation belongs to.

    **Note:** Omit this parameter if you want to create a _Global_
    CompoundAggregation.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DebitReasonListParams"]


class DebitReasonListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    archived: bool
    """Filter using the boolean archived flag.

    DebitReasons can be archived if they are obsolete.

    - TRUE includes DebitReasons that have been archived.
    - FALSE excludes archived DebitReasons.
    """

    codes: List[str]
    """List of Debit Reason short codes to retrieve."""

    ids: List[str]
    """List of Debit Reason IDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Debit Reasons to retrieve per page."""
